package com.greensoft.objectdb.test.console;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.*;

/**
 * http://www.objectdb.com/database/forum/370
 * 
 * 'Why is a mapped LAZY @OneToMany field viewable after the finding entity manger closes,
 *  but an unmapped @OneToMany field is not ?'
 * 
 * IMPORTANT: To run with enhancer on, you can't use -javaagent:lib/objectdb.jar in the Netbeans project properties,
 * because for some reason it then stalls (perhaps because of a clash with JRebel, I am not sure).
 * 
 * Instead go to build/classes and run:
 * 
 * $ java -javaagent:../../lib/objectdb.jar com.greensoft.objectdb.test.console.TestLazyOneToMany
 *
 * The result with DO_VIEW_BEFORE_CLOSE = false is:
 * 
 *  a.listA_mapped:null
 *  a.listB_unmapped:null
 * 
 * Prior to build 2.3.7_05 the result was (see forum support):
 * 
 *  a.listA_mapped:null
 *  a.listB_unmapped:[] 
 * 
 * To compare with not enhanced run:
 * 
 * $ java -cp .:../../lib/objectdb-2.3.7_05.jar com.greensoft.objectdb.test.console.TestLazyOneToMany
 * 
 *  a.listA_mapped:[{A}[4]"ownedElement1", {A}[5]"ownedElement2"]
 *  a.listB_unmapped:null
 * 
 * YOU MAY HAVE TO USE A SPECIFIC OBJECTDB VERSION, CHECK IN LIB
 * 
 * @author Darren Kelly (Webel IT Australia for Greensoft)
 */
public class TestLazyOneToMany {

    /**
     * If true the view/output of the fields of the entities will be called
     * before the relevant finding EntityManager closes.
     */
    final static boolean DO_VIEW_BEFORE_CLOSE = false;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        String $connection = "objectdb:db/TestLazyOneToMany.tmp;drop";
        EntityManagerFactory emf = Persistence.createEntityManagerFactory($connection);

        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();
        A a = new A("aOwner");

        a.listA_mapped = new ArrayList<A>();
        A aOwnedElement1 = new A("ownedElement1");
        aOwnedElement1.a = a;
        a.listA_mapped.add(aOwnedElement1);
        A aOwnedElement2 = new A("ownedElement2");
        aOwnedElement2.a = a;
        a.listA_mapped.add(aOwnedElement2);

        a.listB_unmapped = new ArrayList<B>();
        a.listB_unmapped.add(new B("b1"));
        a.listB_unmapped.add(new B("b2"));

        em.persist(a);
        em.getTransaction().commit();
        Long id = a.id;
        em.close();

        em = emf.createEntityManager();
        a = em.find(A.class, id);

        if (!DO_VIEW_BEFORE_CLOSE) {
            em.close();
        }

        //Output entities and fields

        System.out.println("a.listA_mapped:" + a.listA_mapped);
        //gives: '[a.listA_mapped:[{A}[4]"ownedElement1", {A}[5]"ownedElement2"]
        
        System.out.println("a.listB_unmapped:" + a.listB_unmapped);
        //gives: '[]' or '[{B}[2]"b1", {B}[3]"b2"]'

        if (DO_VIEW_BEFORE_CLOSE) {
            em.close();
        }

        emf.close();
    }

    @Entity
    static public class A {

        @Id
        @GeneratedValue
        Long id;
        String name;

        public A(String name) {
            this.name = name;
        }

        @Override
        public String toString() {
            return "{" + getClass().getSimpleName() + "}" + "[" + id + "]\"" + name + "\"";
        }
        @ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        A a; //maps: listA_mapped
        @OneToMany(fetch = FetchType.LAZY, mappedBy = "a", cascade = CascadeType.ALL)
        List<A> listA_mapped;
        @OneToMany(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
        List<B> listB_unmapped;
    }

    @Entity
    static public class B extends A {

        public B(String name) {
            super(name);
        }
    }
}

/*
From build/classes with objectdb-2.3.7_01:

Without enhancer and DO_VIEW_BEFORE_CLOSE = false;
 
build/classes $ java -cp .:../../lib/objectdb.jar com.greensoft.objectdb.test.console.TestLazyOneToMany 
a.listA_mapped:[{A}[4]"ownedElement1", {A}[5]"ownedElement2"]
a.listB_unmapped:[]

With enhancer and DO_VIEW_BEFORE_CLOSE = false;

build/classes $ java -javaagent:../../lib/objectdb.jar com.greensoft.objectdb.test.console.TestLazyOneToMany
a.listA_mapped:null
a.listB_unmapped:[]

 */